__version__ = "0.1.35"
from pig_ui.ui.ux_element import (
    UXWrapper, 
    UXText, 
    UXRect,
    UXImage,
    UXCircle
)
from pig_ui.ui.ui_element import UIElement
from pig_ui.ui.ui_textinput import UITextInput
from pig_ui.ui.ui_dropdown import UIDropDown
from pig_ui.ui.ui_menu import UIMenuBar
from pig_ui.ui.ui_colorpicker import UIColorPicker
from pig_ui.ui.ui_spinbox import UISpinBox
from pig_ui.ui.ui_sidebar import UISideBar
from pig_ui.ui.constants import Anchors

from pig_ui.ui.ui_manager import UIM
from pig_ui.app import App

from pig_ui.ui.bezier import draw_beziere, compute_bezier_points


def Color(*args) -> tuple[int, int, int, int]:
    r, g, b, a = 0, 0, 0, 255
    if len(args) == 1:
            arg = args[0]
            if isinstance(arg, str):
                hex_str = arg.lstrip('#')
                if len(hex_str) == 6: #252525
                    r = int(hex_str[0:2], 16)
                    g = int(hex_str[2:4], 16)
                    b = int(hex_str[4:6], 16)
                elif len(hex_str) == 8: #252525ff
                    r = int(hex_str[0:2], 16)
                    g = int(hex_str[2:4], 16)
                    b = int(hex_str[4:6], 16)
                    a = int(hex_str[6:8], 16)
                    
            elif isinstance(arg, (tuple, list)):
                if len(arg) >= 3:
                    r, g, b = arg[0], arg[1], arg[2]
                if len(arg) == 4:
                    a = arg[3]
    elif len(args) >= 3:
        r, g, b = args[0], args[1], args[2]
        if len(args) == 4:
            a = args[3]
            
    return (r,g,b,a)
from pygame import Vector2
type VectorLike = int | float | tuple | Vector2
class Vector2: ...
class Vector2:
    def __init__(self, x: float | int, y: float | int):
        self.x, self.y = x, y
    
    @property
    def xx(self) -> Vector2:
        return Vector2(self.x, self.x)
    @property
    def yy(self) -> Vector2:
        return Vector2(self.y, self.y)
    @property
    def yx(self) -> Vector2:
        return Vector2(self.y, self.x)
    @property
    def xy(self) -> Vector2:
        return Vector2(self.x, self.y)
    
    def __add__(self, val: VectorLike) -> Vector2:
        if isinstance(val, (int, float)):
            return Vector2(self.x + val, self.y + val)
        elif isinstance(val, tuple):
            return Vector2(self.x + val[0], self.y + val[1])
        elif isinstance(val, Vector2):
            return Vector2(self.x + val.x, self.y + val.y)
    
    def __sub__(self, val: VectorLike) -> Vector2:
        if isinstance(val, (int, float)):
            return Vector2(self.x - val, self.y - val)
        elif isinstance(val, tuple):
            return Vector2(self.x - val[0], self.y - val[1])
        elif isinstance(val, Vector2):
            return Vector2(self.x - val.x, self.y - val.y)
    
    def __mul__(self, val: VectorLike) -> Vector2:
        if isinstance(val, (int, float)):
            return Vector2(self.x * val, self.y * val)
        elif isinstance(val, tuple):
            return Vector2(self.x * val[0], self.y * val[1])
        elif isinstance(val, Vector2):
            return Vector2(self.x * val.x, self.y * val.y)

    def __truediv__(self, val: VectorLike) -> Vector2:
        if isinstance(val, (int, float)):
            return Vector2(self.x / val, self.y / val)
        elif isinstance(val, tuple):
            return Vector2(self.x / val[0], self.y / val[1])
        elif isinstance(val, Vector2):
            return Vector2(self.x / val.x, self.y / val.y)
    
    def __floordiv__(self, val: VectorLike) -> Vector2:
        if isinstance(val, (int, float)):
            return Vector2(self.x // val, self.y // val)
        elif isinstance(val, tuple):
            return Vector2(self.x // val[0], self.y // val[1])
        elif isinstance(val, Vector2):
            return Vector2(self.x // val.x, self.y // val.y)
    
    def __repr__(self) -> str:
        return f'Vector2<x={self.x},y={self.y}>'
    
    def as_tuple(self) -> tuple:
        return (self.x, self.y)


import json, os
import random
from typing import Callable
from time import time

try:
    import pygame as pg
    from pygame import Rect, Vector2, Surface, Color
    from pygame.surfarray import make_surface
    PG = pg
except:
    def Color(*C): ...
    def Rect(*RV): ...
    def Vector2(*V): ...
    def Surface(*P): ...
    PG = None



from pig_ui.constants import *
from pig_ui.events import Events
from pig_ui.ui.ui_element import UIM

class App:
    def __init__(self, title: str = 'pigUI'):
        self.is_running = True
        self.window = PG.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))
        PG.display.set_caption(title)
        self.clock = PG.time.Clock()
        self.events = Events()

    def run(self):
        while self.is_running:
            self.window.fill((25,25,25))
            self.update()
            UIM.update()
            self.draw()
            self.event_handler()
            
        self.destroy()
    
    def update(self):
        self.clock.tick(60)
    def draw(self):
        if self.events.MOUSE_LEFT:
            PG.draw.circle(self.window,(128,128,0),self.events.MOUSE_POS,8,3)
        if self.events.DOUBLE_CLICK:
            PG.draw.circle(self.window,(0,128,128),self.events.MOUSE_POS,16,3)
        PG.display.update()
        
    def event_handler(self):
        self.events.recv_events()
        if self.events.QUIT:
            self.is_running = False
    def destroy(self):
        self.is_running